import { OmegleStatus, TextClientEvents } from "../types/Omegle.js";
export interface TextClient {
    on<U extends keyof TextClientEvents>(event: U, listener: TextClientEvents[U]): this;
    once<U extends keyof TextClientEvents>(event: U, listener: TextClientEvents[U]): this;
    emit<U extends keyof TextClientEvents>(event: U, ...args: Parameters<TextClientEvents[U]>): boolean;
}
export declare class TextClient {
    private emitter;
    private headers;
    private fetchAgent?;
    private cc;
    private cachedStatus;
    private _randid?;
    private _server?;
    private _session?;
    /**
     * Generate a random ID for the client.
     * @returns A random ID.
     */
    private get randId();
    /**
     * Get a server from omegle's server pool.
     * Omegle returns an array of servers, but based on ./decompOmegle, it seems to select the first one this way.
     * @returns A server from omegle's server pool.
     */
    private get server();
    /**
     * Sets the http agent for all requests.
     * @param httpAgent An optional http agent to use for fetching. Removes the agent if undefined.
     */
    setAgent(httpAgent?: any): void;
    /**
     * Consructs an omegle url.
     * @returns An omegle url.
     */
    get url(): string;
    /**
     * Gets the current omegle status, including usercount, servers, and other info.
     * @returns The OmegleStatus object.
     *
     */
    status(): Promise<OmegleStatus>;
    /**
     * Iterates through all events in the response, emitting them to the client.
     * @param response The response from omegle.
     */
    private manageEvents;
    /**
     * Starts a long-poll loop for the omegle events, breaking when the session is closed.
     */
    private poll;
    /**
     * An antinude measure, this generates a "client cookie" to send with the request that simply verifies the antinude server is running.
     */
    private generateCC;
    /**
     * Sends a chat message.
     * @param message The message content to send.
     *
     * @example
     * await <TextClient>.send("Hello, world!");
     */
    send(message: string): Promise<void>;
    /**
     * Sends a chat typing event, lasting for 2 seconds.
     * @example
     * await <TextClient>.sendTyping();
     */
    sendTyping(): Promise<void>;
    /**
     * Disconnects from the current session, returning if no session is active.
     */
    disconnect(): Promise<void>;
    /**
     * Sends a captcha response back to Omegle.
     * @param answer The captcha solution.
     */
    sendCaptcha(solution: string): Promise<void>;
    /**
     * Starts a new session.
     * @param topics The topics to search for when connecting. (Default none)
     * @param lang The language to use for the chat. (Default 'en')
     */
    connect(topics?: string[], lang?: string): Promise<void>;
}
