"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TextClient = void 0;
const node_fetch_1 = __importDefault(require("node-fetch"));
const events_1 = require("events");
const Utils_js_1 = require("./Utils.js");
class TextClient {
    /*
     Simple event emitter wrapper here, so we can use emit/on/once without exposing the other bloaty functions to the main class.
    */
    emitter = new events_1.EventEmitter();
    on(event, listener) {
        this.emitter.on(event, listener);
        return this;
    }
    emit(event, ...args) {
        return this.emitter.emit(event, ...args);
    }
    once(event, listener) {
        this.emitter.once(event, listener);
        return this;
    }
    /*
        Actual methods here.
    */
    headers = {};
    fetchAgent;
    cc = null;
    cachedStatus = null;
    _randid;
    _server;
    _session;
    /**
     * Generate a random ID for the client.
     * @returns A random ID.
     */
    get randId() {
        if (!this._randid) {
            const set = "23456789ABCDEFGHJKLMNPQRSTUVWXYZ".split("");
            let id = "";
            for (let i = 0; i < 8; i++) {
                id += set[Math.floor(Math.random() * set.length)];
            }
            this._randid = id;
        }
        return this._randid;
    }
    /**
     * Get a server from omegle's server pool.
     * Omegle returns an array of servers, but based on ./decompOmegle, it seems to select the first one this way.
     * @returns A server from omegle's server pool.
     */
    get server() {
        if (!this._server) {
            const n = Math.floor(Math.random() * 48) + 1;
            this._server = `front${n}`;
        }
        return this._server;
    }
    /**
     * Sets the http agent for all requests.
     * @param httpAgent An optional http agent to use for fetching. Removes the agent if undefined.
     */
    setAgent(httpAgent) {
        this.fetchAgent = httpAgent;
    }
    /**
     * Consructs an omegle url.
     * @returns An omegle url.
     */
    get url() {
        return `https://${this.server}.omegle.com/`;
    }
    /**
     * Gets the current omegle status, including usercount, servers, and other info.
     * @returns The OmegleStatus object.
     *
     */
    async status() {
        const url = Utils_js_1.UriBuilder.from({
            uri: this.url + "status",
            query: {
                nocache: Math.random(),
                randid: this.randId
            }
        });
        const status = await (0, node_fetch_1.default)(url, {
            headers: this.headers,
            ...(this.fetchAgent ? { agent: this.fetchAgent } : {})
        }).then((res) => res.json());
        this.cachedStatus = status;
        return status;
    }
    /**
     * Iterates through all events in the response, emitting them to the client.
     * @param response The response from omegle.
     */
    async manageEvents(response) {
        for (const event of response.events) {
            const label = event[0];
            this.emit("raw", event);
            switch (label) {
                case "connected":
                    this._session = response.clientID;
                    this.emit("connect", this._session);
                    this.poll();
                    break;
                case "identDigests":
                    this.emit("digest", event[1].split(","));
                    break;
                case "typing":
                    this.emit("typing");
                    break;
                case "gotMessage":
                    this.emit("message", event[1]);
                    break;
                case "statusInfo":
                    this.emit("status", event[1]);
                    break;
                case "strangerDisconnected":
                    this._session = undefined;
                    this.emit("disconnect");
                    break;
                case "recaptchaRequired":
                    this.emit("captcha", event[1]);
                    break;
                case "waiting":
                    this.emit("waiting");
                    break;
                default:
                    console.log(`Unknown event: ${label}`);
                    break;
            }
        }
    }
    /**
     * Starts a long-poll loop for the omegle events, breaking when the session is closed.
     */
    async poll() {
        while (this._session) {
            const response = await (0, node_fetch_1.default)(this.url + "events", {
                method: "POST",
                headers: {
                    ...this.headers,
                    "content-type": "application/x-www-form-urlencoded; charset=UTF-8"
                },
                body: "id=" + this._session,
                ...(this.fetchAgent ? { agent: this.fetchAgent } : {})
            }).then((res) => res.json());
            if (response == null) {
                this._session = undefined;
                this.emit("disconnect");
            }
            else {
                this.manageEvents({
                    events: response
                });
            }
        }
    }
    /**
     * An antinude measure, this generates a "client cookie" to send with the request that simply verifies the antinude server is running.
     */
    async generateCC() {
        const server = this.cachedStatus?.antinudeservers[Math.floor(Math.random() * this.cachedStatus?.antinudeservers.length)];
        const cc = await (0, node_fetch_1.default)(`https://${server}/check`, {
            headers: this.headers,
            method: "POST",
            ...(this.fetchAgent ? { agent: this.fetchAgent } : {})
        }).then((res) => res.text());
        this.cc = cc;
        return cc;
    }
    /**
     * Sends a chat message.
     * @param message The message content to send.
     *
     * @example
     * await <TextClient>.send("Hello, world!");
     */
    async send(message) {
        if (!this._session) {
            throw new Error("Not in an active session!");
        }
        await (0, node_fetch_1.default)(this.url + "send", {
            method: "POST",
            body: `msg=${encodeURIComponent(message)}&id=${this._session}`,
            headers: {
                "content-type": "application/x-www-form-urlencoded; charset=UTF-8"
            }
        });
    }
    /**
     * Sends a chat typing event, lasting for 2 seconds.
     * @example
     * await <TextClient>.sendTyping();
     */
    async sendTyping() {
        if (!this._session) {
            throw new Error("Not in an active session!");
        }
        await (0, node_fetch_1.default)(this.url + "typing", {
            method: "POST",
            body: `id=${this._session}`,
            headers: {
                "content-type": "application/x-www-form-urlencoded; charset=UTF-8"
            }
        });
    }
    /**
     * Disconnects from the current session, returning if no session is active.
     */
    async disconnect() {
        if (!this._session)
            return;
        await (0, node_fetch_1.default)(this.url + "disconnect", {
            method: "POST",
            body: `id=${this._session}`,
            headers: {
                "content-type": "application/x-www-form-urlencoded; charset=UTF-8"
            }
        });
    }
    /**
     * Sends a captcha response back to Omegle.
     * @param answer The captcha solution.
     */
    async sendCaptcha(solution) {
        await (0, node_fetch_1.default)(this.url + "recaptcha", {
            method: "POST",
            headers: {
                "content-type": "text/plain"
            },
            body: solution
        });
    }
    /**
     * Starts a new session.
     * @param topics The topics to search for when connecting. (Default none)
     * @param lang The language to use for the chat. (Default 'en')
     */
    async connect(topics = [], lang = "en") {
        if (this._session) {
            throw new Error("Already in an active session!");
        }
        if (this.cachedStatus == null) {
            this.cachedStatus = await this.status();
            this.emit("status", this.cachedStatus);
        }
        if (this.cc == null) {
            await this.generateCC();
        }
        const url = Utils_js_1.UriBuilder.from({
            uri: this.url + "start",
            query: {
                caps: "recaptcha2,t3",
                firstevents: 1,
                spid: "",
                randid: this.randId,
                cc: this.cc,
                topics: JSON.stringify(topics),
                lang: lang
            }
        });
        const response = await (0, node_fetch_1.default)(url, {
            method: "POST",
            headers: {
                "Accept": "application/json",
                "dnt": "1",
                "sec-ch-ua": `".Not/A)Brand";v="99", "Google Chrome";v="103", "Chromium";v="103"`,
                "sec-ch-ua-mobile": "?0",
                "sec-ch-ua-platform": "Windows",
                "sec-fetch-dest": "empty",
                "sec-fetch-mode": "cors",
                "sec-fetch-site": "same-site",
                "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36"
            },
            ...(this.fetchAgent ? { agent: this.fetchAgent } : {})
        });
        const omegleResponse = await response.json();
        if (omegleResponse.clientID) {
            this.manageEvents(omegleResponse);
        }
        else {
            this.emit("blocked");
        }
    }
}
exports.TextClient = TextClient;
